from dataEngine import Common, Logger

logger = Logger.get()
ChannelsList = {}
ChannelsListByName = {}


class ChanType:
    SIP = 1
    LOCAL = 2


CHAN_INFO_FIELDS = ("userId", "email", "userExtension", "groupId", "groupName", "phone", "name", "company", "userDepartment")


class Channel(Common.PbxDataObject):
    __slots__ = ("uid", "lid", "userId", "email", "userExtension", "groupId", "groupName", "phone", "name", "company",
                 "chanType", "chanName", "context", "removed", "userDepartment")

    def __init__(self, uid, lid, userID, email, userExtension, groupID, groupName, phone, name, chanType, chanName, context, userDepartment):
        self.uid = uid
        self.lid = lid
        self.userId = userID
        self.email = email
        self.userExtension = userExtension
        self.groupId = groupID
        self.groupName = groupName
        self.phone = phone
        self.name = name
        self.company = None
        self.chanType = chanType
        self.chanName = chanName
        self.context = context
        self.removed = False
        self.userDepartment = userDepartment

    def __str__(self):
        return "%s(%s) - %s (%s)" % (self.uid, self.lid, self.chanName, self.get_info())

    def get_info(self):
        return {attr:self.get(attr) for attr in CHAN_INFO_FIELDS if self.get(attr) is not None}

    def set_info(self, info):
        for key, value in info.items():
            self.set(key, value)

    def isRemoved(self):
        return self.removed

    def __eq__(self, other):
        return all(other and self.get(p) == other.get(p) for p in CHAN_INFO_FIELDS)


def add(chan, pbx="callweaver"):
    if pbx not in ChannelsList:
        ChannelsList[pbx] = {}
        ChannelsListByName[pbx] = {}
    ChannelsList[pbx][chan.uid] = chan
    ChannelsListByName[pbx][chan.chanName] = chan


def get(uid, pbx="callweaver"):
    if pbx in ChannelsList:
        return ChannelsList[pbx].get(uid)


def get_by_name(chanName, pbx="callweaver"):
    if pbx in ChannelsList:
        return ChannelsListByName[pbx].get(chanName)


def uid_masquerade(uid1, uid2, lid, pbx="callweaver"):
    chan1 = ChannelsList[pbx].get(uid1)
    chan2 = ChannelsList[pbx].get(uid2)
    if chan1 is None or chan2 is None:
        return
    chan1.set("uid", uid2)
    chan1.set("lid", lid)
    chan2.set("uid", uid1)
    chan2.set("lid", lid)
    ChannelsList[pbx][uid1] = chan2
    ChannelsList[pbx][uid2] = chan1
    ChannelsListByName[pbx][chan1.chanName] = chan2
    ChannelsListByName[pbx][chan2.chanName] = chan1


def end(uid, pbx="callweaver"):
    if pbx in ChannelsList:
        chan = ChannelsList[pbx].pop(uid, None)
        if chan:
            ChannelsListByName[pbx].pop(chan.chanName, None)


def print_current_state():
    logger.notice(get_current_state())


def end_all():
    for pbx in ChannelsList:
        for chan in ChannelsList[pbx].keys():
            end(chan, pbx)


def get_current_state():
    lines = ["All channels details:\r\n"]
    for pbx in ChannelsList:
        lines.append("%s PBX: %s" % ("=" * 10, pbx))
        for channel in ChannelsList[pbx]:
            lines.append("----- " + str(ChannelsList[pbx][channel]))
    lines.append("\r\n")
    return "\r\n".join(lines)
