from dataEngine import AppConfig, Logger

logger = Logger.get()

CON = None
MOCK = None
PcapReader = None


class ConnectorType():
    def __init__(self):
        pass
    CwConnector = 1
    FsConnector = 2


def get_connector_name(type):
    if type == ConnectorType.CwConnector:
        return "cw"
    if type == ConnectorType.FsConnector:
        return "fs"
    return "unknown"


def get_connector_type(name):
    if name == "cw":
        return ConnectorType.CwConnector
    if name == "fs":
        return ConnectorType.FsConnector
    return None


class BaseConnector():
    def __init__(self):
        pass

    def close_connection(self):
        pass

    def init_connection(self):
        pass

    def get_my_port(self):
        pass

    def send_action(self, action):
        pass


class BaseMock():
    def __init__(self):
        pass

    def init_connection(self):
        pass

    def get_logged_state(self):
        pass

    def send(self, data):
        pass

    def close_connection(self):
        pass


class BasePcapReader():
    def __init__(self):
        pass

    def init(self, data):
        pass

    def close(self):
        pass


def get_conn():
    return CON


def get_mock():
    return MOCK


def get_pcap_reader():
    return PcapReader


def init(type):
    ConName = get_connector_name(type)
    global CON
    CON = __import__("dataEngine.{}.Connector".format(ConName), fromlist=[ConName, 'Connector']).Connector()

    if AppConfig.isTest():
        global MOCK
        try:
            MOCK = __import__("dataEngine.{}.Mock".format(ConName), fromlist=[ConName, 'Mock']).Mock()
        except ModuleNotFoundError:
            MOCK = BaseMock()
            logger.warning("Module dataEngine.{}.Mock has not implemented yet".format(ConName))

        global PcapReader
        try:
            PcapReader = __import__("dataEngine.{}.PcapReader".format(ConName), fromlist=[ConName, 'PcapReader']).PcapReader()
        except ModuleNotFoundError:
            PcapReader = BasePcapReader()
            logger.warning("Module dataEngine.{}.PcapReader has not implemented yet".format(ConName))
